#!/usr/bin/perl
use strict;
use lib './lib';
use Web::Authenticate::User::Storage::Handler::SQL;
use Web::Authenticate::User;
use DBIx::Raw;

my $dbix_raw = DBIx::Raw->new(dsn => 'dbi:mysql:test:127.0.0.1:3306', user => 'root', password => 'forget1');
my $storage_handler = Web::Authenticate::User::Storage::Handler::SQL->new(dbix_raw => $dbix_raw);

#my $user = $storage_handler->store_user('admin', 'password');
my $user = $storage_handler->load_user('admin', 'password');

print $user->id;

print "\n";
